# coding: utf-8
"""
    AtsasTools.py

    Copyright (c) 2016-2018, Masatsuyo Takahashi, KEK-PF
"""
import sys
import os
import glob
import re
import numpy                as np
from OurSubprocess          import exec_subprocess
import codecs
from IntensityData          import IntensityData
from Guinier                import Guinier
from Result                 import Result
from ErrorResult            import ErrorResult
from Settings               import get_setting, set_setting

autorg_exe_array = []

def set_exe_array():
    atsas_dir = glob.glob( 'C:/atsas*' )
    for dir_ in atsas_dir:
        for sub_dir in [ '/bin', '' ]:
            exe_path = dir_ + sub_dir + '/autorg.exe'
            if os.path.exists( exe_path ):
                autorg_exe_array.append( exe_path )
                break

    atsas_exe_paths = get_setting( 'atsas_exe_paths' )
    if len( autorg_exe_array ) > 0:
        set_setting( 'atsas_exe_paths', autorg_exe_array )

    # print( 'autorg_exe_array=', autorg_exe_array )

set_exe_array()

def autorg( file, exe_index=0, out_file=None, smaxrg=1.3 ):
    # print( 'autorg: file=', file )
    assert( os.path.exists( file ) )
    if len(autorg_exe_array) == 0:
        return ErrorResult(), None

    exe_ = autorg_exe_array[ exe_index ]
    opts = ' --smaxrg ' + str(smaxrg)

    if out_file is not None:
        opts += ' -o ' + out_file

    cmd = '%s %s %s -f csv' % ( exe_, opts, file )
    out, err = exec_subprocess( cmd, shell=True )
    # print( out, err )
    if err == '':
        lines = out.split( '\n' )
        if len( lines ) >= 3:
            intensity = IntensityData( file )
            x, y, e = intensity.get_guinier_valid_xy()

            File, R, RSD, I, ISD, F, L, Q, A = lines[1].split( ',' )
            print( R, RSD, I, ISD, F, L, Q, A )
            f, t = int(F)-1, int(L)-1
            f_, t_ = list( map( lambda i: intensity.positive_index[i], ( f, t ) ) )
            Rg = float( R )
            minQ = np.sqrt( x[f_] )
            maxQ = np.sqrt( x[t_] )

            orig_result = Result(
                            type='A',
                            Rg=Rg, Rg_stdev=float( RSD ),
                            I0=float(I), I0_stdev=float(ISD),
                            From=f_, To=t_,
                            Quality=float(Q),
                            Aggregated=float(A),
                            min_qRg=minQ * Rg,
                            max_qRg=maxQ * Rg,
                            min_curvature=None,
                            max_curvature=None,
                            )
            try:
                guinier = Guinier( x, y, e, positive_ratio=intensity.positive_ratio )
                guinier.estimate_rg( [ f_, t_ ], [ 0, 1.3 ] )
                eval_result = guinier.get_result()
            except:
                # as in 20180204/Open01/Open01_00171_sub.dat
                eval_result = None

            return orig_result, eval_result
        else:
            return ErrorResult(), None

    else:
        print( err )
        return ErrorResult(), None
