# coding: utf-8
"""

    ProgressMinDialog.py

    Copyright (c) 2018, Masatsuyo Takahashi, KEK-PF

"""
from OurTkinter import Tk, Dialog, ttk
from TkSupplements import set_icon

class ProgressMinDialog(Dialog):
    def __init__(self, parent, title="ProgressMinDialog", message=None, num_steps=10, length=200,
                    progress_cb=None, geometry_info=None, cancelable=False, interval=500):
        self.parent = parent
        self.num_steps = num_steps
        self.length = length
        self.canceled = False
        self.message = message
        self.progress_cb = progress_cb
        self.geometry_info = geometry_info
        self.cancelable = cancelable
        self.interval = interval
        auto_geometry = geometry_info is None
        Dialog.__init__( self, self.parent, title, auto_geometry=auto_geometry )

    def body(self, body_frame):   # overrides Dialog.body
        set_icon(self)
        """
            https://stackoverflow.com/questions/7310511/how-to-create-downloading-progress-bar-in-ttk
        """
        if self.message is not None:
            label = Tk.Label(self, text=self.message)
            label.pack()

        self.mpb = ttk.Progressbar(body_frame,orient ="horizontal", length=self.length, mode="determinate")
        self.mpb.pack(padx=5, pady=5)
        self.mpb["maximum"] = self.num_steps
        self.update()

        if self.geometry_info is not None:
            self.geometry(self.geometry_info)
            self.update()

        self.step = 0
        self.refresh()

    def buttonbox( self ):
        box = Tk.Frame(self)
        box.pack()
        if self.cancelable:
            w = Tk.Button(box, text="Cancel", width=10, command=lambda: self.cancel(ask=True) )
            w.pack(side=Tk.LEFT, padx=5, pady=5)

    def cancel(self, ask=False):
        if ask:
            import CustomMessageBox as MessageBox
            yn = MessageBox.askyesno( "Cancel comfirmation",
                "Are you sure to cancel this process?", parent=self )
            if yn:
                self.canceled = True
            else:
                return

        super(ProgressMinDialog, self).cancel()

    def refresh(self):
        self.mpb["value"] = self.step
        if self.progress_cb is None:
            self.step += 1
        else:
            self.step = self.progress_cb()

        if self.step >= self.num_steps:
            self.ok()
        else:
            self.after( self.interval, self.refresh )
            self.update()
