# coding: utf-8
"""

    ファイル名：   TkUtils.py

    処理内容：

    Copyright (c) 2016-2018, Masatsuyo Takahashi, KEK-PF

"""
import os
import sys
import re

PYTHON_DEMO_MONITOR     = 'PYTHON_DEMO_MONITOR'
DO_NO_ADJUST_GEOMETRY   = 'DO_NO_ADJUST_GEOMETRY'

def split_geometry( geometry ):
    n = geometry.split('+')
    s = n[0].split('x')
    w = int( s[0] )
    h = int( s[1] )
    x = 0 if len(n) < 2 else int( n[1] )
    y = 0 if len(n) < 3 else int( n[2] )
    return [ w, h, x, y ]

def join_geometry( w, h, x, y ):
    return '%dx%d+%d+%d' % ( w, h, x, y )

def geometry_fix( top, x, y ):
    W, H, X, Y = split_geometry( top.geometry() )
    top.geometry( join_geometry( W, H, x, y ) )
    top.update()

"""
    URL: http://effbot.org/tkinterbook/wm.htm
"""
def parsegeometry(geometry):
    m = re.match("(\d+)x(\d+)([-+]\d+)([-+]\d+)", geometry)
    if not m:
        raise ValueError("failed to parse geometry string")
    return map(int, m.groups())

def convert_to_tkgeometry( monitor ):
    return re.sub( r'monitor\((.+)\)', '\g<1>', str( monitor ) )

monitors = None
max_monitor = None

def get_monitor_list():
    global monitors
    if monitors is None:
        try:
            from screeninfo import get_monitors
            monitors = get_monitors()
        except:
            print( 'screeninfo is not installed.' )
            monitors = []
    return monitors

def get_max_monitor():
    global monitors, max_monitor

    if max_monitor is not None: return max_monitor

    monitors = get_monitor_list()

    for m in monitors:
        # print( 'get_max_monitor', str( m ) )
        if max_monitor is None or m.width + m.height > max_monitor.width + max_monitor.height:
            max_monitor = m

    return max_monitor

do_not_adjust_geometry = os.environ.get( DO_NO_ADJUST_GEOMETRY )
"""
    TODO:
    some tests seem to go wrong with adjusted geometry.
    it may have been improved by self.udate() before applying adjusted geometry.
"""

def get_selected_monitor():
    monitor = get_max_monitor()
    demo_monitor = os.environ.get( PYTHON_DEMO_MONITOR )
    # print( 'demo_monitor=', demo_monitor )
    if demo_monitor is not None:
        monitor = monitors[ int(demo_monitor) ]
        # print( 'selected monitor=', monitor )
    return monitor

def adjusted_geometry( geometry, monitor=None ):
    global max_monitor, monitors

    if do_not_adjust_geometry:
        return geometry

    if monitor is None:
        monitor = get_selected_monitor()

    try:
        w0, h0, x0, y0 = split_geometry( geometry )
        w1, h1, x1, y1 = monitor.width, monitor.height, monitor.x,  monitor.y
        if w1 > w0: w1 = w0
        if h1 > h0: h1 = h0
        return join_geometry( min(w0, w1), min(h0, h1), x1, y1 )
    except:
        return geometry

def geometry_move( toplevel, parent, x=50, y=50 ):
    x_, y_ = (parent.winfo_rootx()+x, parent.winfo_rooty()+y)
    # print( 'geometry_move: x_, y_=',  x_, y_  )
    toplevel.geometry("+%d+%d" % ( x_, y_ ) )

def is_low_resolution():
    monitor = get_selected_monitor()
    # print( monitor.width, monitor.height )
    return monitor.height < 800
