# coding: utf-8

import sys
import os
import re
import time
import socket
from nose.tools             import eq_
from subprocess             import call, Popen

this_dir = os.path.dirname( os.path.abspath( __file__ ) )
up_dir  = os.path.dirname( this_dir ).replace( '\\', '/' )
exe_dir = up_dir + '/executables'

sys.path.append( this_dir + '/../lib' )
m = re.search( r'autorg_kek-(\d+)_(\d+)_(\d+)', this_dir )
ver_num = int(m.group(1))*10000 + int(m.group(2))*100 + int(m.group(3))
if ver_num > 610:
    import KekLib, AutorgKek
else:
    import CommandMain
from BasicUtils             import clear_dirs_with_retry
from CommandMain            import autorg_func
from ClientServer           import AutorgKekClient

sys.path.append( this_dir + '/../..' )
from TestEnv import env_dict, get_data_folder

def test_load():
    assert True

def test_setup( start=True ):
    global test_params, out_dir

    test_params = []

    in_file1    = env_dict[ 'in_folder3' ] + '/GI/GI_0.7_ave_sub.dat'
    in_file2    = env_dict[ 'in_folder_error_3' ] + '/AIMdim01_00276_sub.dat'
    out_dir     = ( this_dir + '/temp_csharp' ).replace( '\\', '/' )

    test_params = [
        [ in_file1, out_dir + '/result_kek.csv',                0, 0,  0 ],
        [ in_file1, out_dir + '/result_stsas.csv',              1, 0,  0 ],
        [ in_file2, out_dir + '/result_kek_error.csv',          0, 0,  0 ],
        [ in_file2, out_dir + '/result_kek_error_robust.csv',   0, 1,  0 ],
        [ '-v', None, 0, 0, 0 ],
        ]

    if start:
        start_server()

def do_CommandMain( folder, file, out_file, subprocess=True, atsas=False, fh=None ):
    global argv, run_dir
    path    = folder + '/' + file

    if subprocess:
        exe_name = 'autorg_kek.bat'
        atsas_arg = '-A' if atsas else ''
        ret = call( '%s/%s %s -o %s %s' % ( run_dir, exe_name, atsas_arg, out_file, path ), shell=True )
        # eq_( ret, 0 )
    else:
        os.chdir( run_dir )
        sys.argv = argv + [ '-o', out_file, path ]
        command = CommandMain()
        result = command.execute( path, out_file, robust=True )
        rec = ','.join( [ folder, file ]
                        + list( map( lambda x: '%g' % x, result.quality_object.get_factors() + [ result.fit.result.aic ] ) )
                      )
        fh.write( rec + '\n' )

def test_autorg_func():
    global test_params, out_dir
    clear_dirs_with_retry( [ out_dir ] )

    for in_file, out_file, atsas, robust, expected_ret in test_params:
        if in_file == '-v':
            continue
        ret = autorg_func( in_file, out_file, atsas, robust )
        eq_( ret, expected_ret )
        if ret == 0:
            assert( os.path.exists( out_file ) )

def test_Client():
    global test_params

    client = AutorgKekClient()

    for in_file, out_file, atsas, robust, expected_ret in test_params:
        if in_file == '-v':
            retcode = 0
        else:
            args = ( in_file, out_file )

            if atsas:
                args += ( '--atsas', )
            if robust:
                args += ( '--robust', )
            args += ( 'EXECUTE', )
            retcode = client.request( *args )

        eq_( retcode, expected_ret )

def do_client_exe( test_params ):
    global out_dir
    clear_dirs_with_retry( [ out_dir ] )

    # client_exe = up_dir + '/executables/autorg_kek_client.exe'
    client_exe = up_dir + '/autorg_kek_client.bat'

    for in_file, out_file, atsas, robust, expected_ret in test_params:
        exe_args = [ client_exe ]

        if in_file == '-v':
            exe_args.append( '-v' )
            ret = call( ' '.join( exe_args ), shell=True )
            eq_( ret, abs(expected_ret) )
        else:
            if atsas == 1:
                exe_args.append( '-A' )
            if robust == 1:
                exe_args.append( '-r' )
            exe_args += [ '-o', out_file, in_file ]

            # print( 'exe_args=', exe_args )

            ret = call( ' '.join( exe_args ), shell=True )
            eq_( ret, abs(expected_ret) )
            if ret == 0:
                assert( os.path.exists( out_file ) )

def test_client_exe_basic():
    do_client_exe( test_params )

def do_client_exe_all_in_a_folder( in_folder, name=None, check_ret=True ):
    clear_dirs_with_retry( [ out_dir ] )

    client_exe = up_dir + '/autorg_kek_client.bat'

    for root, dirs, files in os.walk( in_folder ):
        for file in files:
            if file.find( '.dat' ) < 0: continue
            if name is not None and file != name: continue

            in_file     = root + '/' + file
            out_file    = out_dir + '/' + file.replace( '.dat', '.csv' )
            print( 'out_file=', out_file )
            exe_args    = [ client_exe, '-o', out_file, in_file ]

            ret = call( ' '.join( exe_args ), shell=True )
            if check_ret:
                eq_( ret, 0 )
                assert( os.path.exists( out_file ) )

def test_client_exe_all_normal( name=None ):
    global out_dir
    in_folder   = env_dict[ 'in_folder3' ]
    do_client_exe_all_in_a_folder( in_folder, name=name )

def test_client_exe_encoding( name=None ):
    global out_dir
    in_folder   = get_data_folder('20190416you')
    do_client_exe_all_in_a_folder( in_folder, name=name )

def tttt_client_exe_all_error( name=None ):
    global out_dir
    in_folder   = env_dict[ 'in_folder_error' ]
    do_client_exe_all_in_a_folder( in_folder, name=name, check_ret=False )

def tttt_client_exe_all_error_2( name=None ):
    global out_dir
    in_folder   = env_dict[ 'in_folder_error_2' ]
    do_client_exe_all_in_a_folder( in_folder, name=name, check_ret=False )

def tttt_Report( name=None ):
    global out_dir
    data_folder = env_dict[ 'in_folder_conc' ] + '/OA01/Backsub'
    conc_folder = env_dict[ 'in_folder_conc' ] + '/OA01/spectra'

    client = AutorgKekClient()
    client.request_impl( b'SERIAL_ANALYZER' )

    client.request_impl( b'PROCESS_ARGS' )

    args = ( data_folder, conc_folder, out_dir )

    client.request( *args )

    num_steps =  client.request_impl( b'START_PROCESS' )
    print( 'num_steps=', num_steps )
    # eq_( num_steps, 0 )

    done = False

    for i in range(3):
        step_num = client.request_impl( b'REQUEST_INFO' )

    client.request_impl( b'CANCEL_PROCESS' )

    # client.request_impl( b'PROCESS_ARGS' )
    # client.request( *args )

    client.request_impl( b'START_PROCESS' )

    num_steps =  int( client.request_impl( b'GET_NUM_STEPS' ).decode() )
    print( 'num_steps=', num_steps )

    # while not done:
    for i in range( num_steps ):
        step_num = client.request_impl( b'REQUEST_INFO' )
        if step_num == 0:
            break

def test_teardown():
    stop_server()

def start_server( auto=True ):
    # exe_file = exe_dir + '/autorg_kek.exe'
    exe_file = up_dir + '/autorg_kek_client.bat'
    print( exe_file )
    ret = call( '%s -S ' % exe_file, shell=True )
    time.sleep( 3 )

def stop_server( auto=True ):
    # client = AutorgKekClient()
    # client.stop_server()
    exe_file = up_dir + '/autorg_kek_client.bat'
    print( exe_file )
    ret = call( '%s -P ' % exe_file, shell=True )

if __name__ == '__main__':
    # test_setup( start=False )
    test_setup( start=True )
    try:
        # test_autorg_func()
        # test_client_exe_basic()
        # test_client_exe_all_normal()
        test_client_exe_encoding()
        # tttt_client_exe_all_error()
        # tttt_client_exe_all_error_2()
        # test_client_exe_all_error_3()
        # test_Client()
        # tttt_Report()
    except:
        pass
    test_teardown()
